////////////////////////////////////////////////////////////////////////////////
//
//    List Manager - Header file  --- list.h
//
//  (c) Copyright 2003 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////

{ .#ifndef _LIST_H_INCLUDED_}
{ .#define _LIST_H_INCLUDED_}

///**
//	@addtogroup list
//	@{
//*/

///**	@file
//	List Header file.
//	Declarations and inlines functions.
//*/

type
  PBListItem = ^TBListItem;
  PSListItem = ^TSListItem;
///**
//	Basic List Item
//*/
  TBListItem = record
	Next: PBListItem;
	Prev: PBListItem;
	SLIt: PSListItem
    Data: Pointer;
  end;

{ .#define BLISTITEM(o) ((PBListItem)(o))}

///**
//	Simple List Item 
//*/
type 
  TSListItem = record
	o: TBListItem;
	Link: PBListItem;     //hinzugefgt am 20.01.2016, war voher -> Data: Pointer
  end;

{ .#define SLISTITEM(o) ((PSListItem)(o))}

///**
//	Basic and Simple List (identical) 
//*/
type 
  PBList = ^TBList
  TBList = record
	Last: PBListItem;
	Link: PSListItem;     //hinzugefgt am 20.01.2016
  end; 
  PSList = PBList;
  FreeProc = procedure (var Data: Pointer);

{ .#define BLIST(o) ((PBList)(o))}

procedure BListRemoveItemEx ( o: PBList; i: PBListItem );
function SListAdd ( o: PBList; var Data ): PSListItem;
function BListAddItem ( o: PBList; i: PBListItem ): PBListItem;
procedure FreeSList ( o: PBList; FreeData: FreeProc );
function NewBList ( ): PBList;
function SListFoundItem ( o: PBList; var Data ): PSListItem;


function SListFoundItem ( o: PBList; var Data: Pointer ): PSListItem; inline;
var a,b: PBListItem; c: PSListItem;
begin
  if not Assigned( o^.Last ) then SListFoundItem := NIL;
  a := o^.Last^.Next;
  b := o^.Last^.Next;
  if Assigned( a ) do 
  while a<>nil do {vorher a<>b}
  begin
	if ( a^.Data = Data ) then begin SListFoundItem := a; break; end;
	a := a^.Next;
  end; 
  SListFoundItem := NIL;
end;

function NewBList ( ): PBList; inline;
var o: PBList;
begin
  New(o);
  o^.Last = NIL;
  NewBList := o;
end;

procedure FreeSList ( var o: PBList; FreeData: FreeProc ); inline;
var a,b,n: PBListItem;
begin
	if Assigned( o^.Last ) then
	begin
		a := o^.Last;
        b := o^.Last;
		while a<>nil do {vorher a<>b}
        begin 
			n := a^.Prev;
			//if ( FreeData && SLISTITEM(a)->Data ) FreeData(SLISTITEM(a)->Data);
			if Assigned(FreeData) then if Assigned(a^.Data) then FreeData(a^.Data);
			Dispose(a);
			a := n;
		end;
	end;
	Dispose(o);
end;

function BListAddItem ( o: PBList; i: PBListItem ): PBListItem; inline;
begin
	if Assigned( o^.Last ) then
	begin
		i^.Next := o^.Last^.Next;
		i^.Prev := o^.Last;
		o^.Last^.Next^.Prev := i;
		o^.Last^.Next := i;
		o^.Last := i;
	end else 
	begin
		i^.Next = i;
		i^.Prev = i;
		o^.Last = i;
	end;
	BListAddItem := i;
end;

function SListAdd ( o: PBList; var Data ): PSListItem; inline;
begin	
  PSListItem i = (PSListItem)malloc(sizeof(TSListItem));
  i^.Link^.Data := Pointer(Data);
  o^.Last^.SLIt := i;
  //BListAddItem(o,BLISTITEM(i));
    if Assigned( o^.Last ) then
  begin
	i^.Link^.Next := o^.Last^.Next;
	i^.Link^.Prev := o^.Last;
	o^.Last^.Next^.Prev := i^.Link;
	o^.Last^.Next := i^.Link;
	o^.Last := i^.Link;
  end else 
  begin
	i^.Link^.Next = i^.Link;
	i^.Link^.Prev = i^.Link;
	o^.Last = i^.Link;
  end;
  SListAdd := i;
end;

_INLINE void BListRemoveItemEx ( PBList o, PBListItem i ) {
  if ( i->Next == i ) {
	  o->Last = NULL;
  } else {
	  i->Next->Prev = i->Prev;
	  i->Prev->Next = i->Next;
	  if ( o->Last == i ) o->Last = i->Prev;
  }
}

typedef struct TListItem *PListItem;
typedef struct TListItem
{
	PListItem Next;
	PListItem Prev;
	void *Data;
	
	l_ulong DataType;
	char *Key;
	void (*FreeData) (void*);
} TListItem;

typedef struct TList *PList;
typedef struct TList
{
	PListItem Last;
	PListItem Current;
} TList;

PListItem  NewListItemEx   ( char *Key, void *Data, void (*FreeData) (void*), l_ulong DataType, l_ulong DataSize );
PListItem  NewListItem     ( char *Key, void *Data, void (*FreeData) (void*) );
PListItem  ListAddItem     ( PList o, PListItem i );
PListItem  ListAdd         ( PList o, char *Key, void *Data, void (*FreeData) (void*) );
char       ListRemoveItem  ( PList o, PListItem i );
PListItem  ListAtItem      ( PList o, unsigned long Index );
void      *ListAt          ( PList o, unsigned long Index );
PListItem  ListKeyItem     ( PList o, char *Key );
void      *ListKey         ( PList o, char *Key );
PListItem  ListKeyCaseItem     ( PList o, char *Key );
void      *ListKeyCase         ( PList o, char *Key );
PListItem  ListFoundItem   ( PList o, void *Data );
PList      NewList         ( void );
unsigned long ListFoundIndex ( PList o, void *Data );
PListItem ListAddItemBefore ( PList o, PListItem i, PListItem b );
char ListRemoveItemEx ( PList o, PListItem i );

// /!\ This 2 functions dot no order Items in List, but Data /!\ ->-> To be improved ...
char       ListOrderByData ( PList o, long (*comparator) ( void*,void*) );
char       ListOrderByItem ( PList o, long (*comparator) ( PListItem,PListItem) );

// free list, all items and data. Ideal for most uses.
void FreeList ( PList o );

// Same as freelist but do not free data. (skip DataFree)
void KickList ( PList o );

#endif /* _LIST_H_INCLUDED_ */
