/////////////////////////////////////////////////////////////////////////////////
//
//	Desktop
//
//	Aura Milestone 4 
//
//	(c) Copyright 2013 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////

#include "kernel.h"
#include "internal.h"
#include "widget.h"
#include "button.h"
#include "label.h"
#include "canvas.h"
#include "menu.h"
#include "list.h"
#include "skin.h"
#include "window.h"
#include "fms2.h"
#include "sound.h"
#include "Desktop/desktop.h" 
#include <math.h>

l_ulong AppVersion = ULONG_ID(5,0,1,8);
char    AppName[] = "Desktop";
l_uid   nUID = "app:desktop";
l_uid NeededLibs[] = { "widget", "skin","button","label","menu","fms2","window","grfx","treeview","iodlg","mp3","sound""" };

PSStream stream = NULL;
PSStream stream2 = NULL;
l_bool LoggedIn						= false;
l_bool AppEventHandler ( PWidget o, PEvent Ev );

// Include the desktop files	//
#include "Desktop/theme.c"
#include "Desktop/wallpaper.c"
#include "Desktop/panels.c"

#define IDM_doslynx		0xFFFF9902

#define IDM_ftp 		0xFFFF9903

#define IDM_dillo		0xFFFF9904

l_text Repeat = 0;

void GenerateSubMenu ( PMenuItem o ,void* Args) {
        l_text n; p_bitmap i16; l_ulong  fi;

        PMenuItem m = NULL;
        PListItem a, b;
        PList l;


        l = ListDir(Args,0);
        if ( l ) {
                if ( l->Last )
                {
                        a = b = l->Last;
                        if ( a )
                        do
                        {
                                if ( FILEINFO(a->Data)->Name[0] != '.' ){
                                        if ( TextRChr(FILEINFO(a->Data)->Name,'.') )
                                                n = TextNDup(FILEINFO(a->Data)->Name,TextRChr(FILEINFO(a->Data)->Name,'.')-FILEINFO(a->Data)->Name);
                                        else
                                                n = TextDup(FILEINFO(a->Data)->Name);
                                        //FileGetIcons2(FILEINFO(a->Data),&i16,NULL,NULL,&fi);
                                        i16 = FileGetIcon2(FILEINFO(a->Data),16,&fi);
                                        m = NewMenuItemEx(n,i16,IDM_PROGRAMS,fi?MIF_FREEICON:0,NULL,NULL,TextDup(FILEINFO(a->Data)->VPathName),m );
                                        free(n);
                                }

                                a = a->Prev;
                        }
                        while ( a != b );
                }
                FreeList(l);
        }

        l = ListDir(Args,FA_DIREC|FA_LABEL);

        if ( l ) {;
                if ( l->Last )
                {
                        a = b = l->Last;
                        if ( a )
                        do
                        {
                                if ( FILEINFO(a->Data)->Attributes & FA_DIREC ) {
                                        //FileGetIcons2(FILEINFO(a->Data),&i16,NULL,NULL,&fi);
                                        i16 = FileGetIcon2(FILEINFO(a->Data),16,&fi);
                                        m = NewMenuItemEx(FILEINFO(a->Data)->Name,i16,0,0,NULL,&GenerateSubMenu,TextDup(FILEINFO(a->Data)->VPathName),m );
                                }
                                a = a->Prev;
                        }
                        while ( a != b );
                }
                FreeList(l);
        }

        o->SubMenu = NewMenu(m);

}


void DesktopWidgetInsert ( PWidget o, PWidget i )
{
        _WidgetInsert(o,i);

        if ( WidgetIsClassId(i,ULONG_ID('W','i','n',' ')) )
        {
                DebugMessage("TaskbarAdd %x",i);
                TaskbarAdd(WINDOW(i));
        }
}

void DesktopWidgetSetFirstEx ( PWidget o, PWidget i  )
{
        _WidgetSetFirstEx(o,i);

        if ( WidgetIsClassId(i,ULONG_ID('W','i','n',' ')) )
        {
                if (WINDOW(i)->WindowStates & WS_MINIMIZED)
                {
                        WINDOW(i)->WindowStates &=~ WS_MINIMIZED;
                }

                TaskbarRedraw(WINDOW(i));
        }
}


void DesktopWidgetRemove ( PWidget o, PWidget i )
{
        _WidgetRemove(o,i);

        if ( WidgetIsClassId(i,ULONG_ID('W','i','n',' ')) )
        {
                DebugMessage("TaskbarRemove %x...",i);
                TaskbarRemove(WINDOW(i));
                DebugMessage("...finished");
        }
}

//	Global variables	//
PMenu Menu                      = 0;
PMenu Menu2                      = 0;
PMenu Menu3                      = 0;
PMenu VolTraybarMenu		= 0;
PTimer  ClockTimer      = 0;
PTimer  IconsTimer      = 0;
PTimer  EndTimer      = 0;
PWidget VOLEventHandler = 0;
l_int VolumeMute	= 0;



l_bool VolumeEventHandler ( PWidget o, PEvent Event )
{
	if ( Event->Type == EV_MESSAGE )
	{
		switch ( Event->Message )
		{
                case IDM_VOL_ADJ:
                {
                        FileRun("applications/setvol.app", NULL);
                        return true;
                }
                break;

		case IDM_MUTE:
                {
			VolumeDigi = KeyGetInt("/USER/SOUND/VOLUMEDIGI",255);
			VolumeMidi = KeyGetInt("/USER/SOUND/VOLUMEMIDI",255);
			VolumeMute = KeyGetInt("/USER/SOUND/VOLUMEMUTE",0);
			if (VolumeMute == 0)
			{
				VolumeMute = 1;
				KeySetInt("/USER/SOUND/VOLUMEMUTE",VolumeMute);
				set_volume(0,0);
			}
			else
			{
				VolumeMute = 0;
				KeySetInt("/USER/SOUND/VOLUMEMUTE",VolumeMute);
				set_volume(VolumeDigi,VolumeMidi);
			}
                        return true;
                }
                break;
		}
	}

	return false;
}

void VolumeInit ( void )
{
	VolumeDigi = KeyGetInt("/USER/SOUND/VOLUMEDIGI",255);
	VolumeMidi = KeyGetInt("/USER/SOUND/VOLUMEMIDI",255);
	VolumeMute = KeyGetInt("/USER/SOUND/VOLUMEMUTE",0);
	if (VolumeMute == 0)
		set_volume(VolumeDigi,VolumeMidi);
	else
		set_volume(0,0);
}

void VolumeTrayInit ( void )
{
	TRect r;
	RectAssign(&r,0,0,0,0);
	VOLEventHandler = CreateWidget(&Me, r);
	VOLEventHandler->AppEvHdl = &VolumeEventHandler;

	PMenu VolTraybarMenu =
	 NewMenu(
	  NewMenuItem("Adjust Volume", NULL, IDM_VOL_ADJ, NULL, NULL,
	  NewMenuItem("Mute/UnMute Volume", NULL, IDM_MUTE, NULL, NULL, NULL
	  ))
	 );

	TraybarAdd(LoadImage("SYSTEM/ICONS/vol.bmp"), "Adjust Volume", VolTraybarMenu, VOLEventHandler);
}

void Authenticate ( void )
{
	LoggedIn = TRUE;
}

PWindow w	= 0;

l_bool AppEventHandler ( PWidget o, PEvent Ev )
{
        switch (Ev->Message)
        {
		    case BTN_APP:
                {
                        TPoint p;
                        PointAssign(&p,WIDGET(GSApp)->Absolute.a.x-5, WIDGET(GSApp)->Absolute.b.y-33);
                        PopUpMenu(&Me, p, Menu2, WIDGET(GSApp), 1);
                        return true;
                }
                break;

		    case BTN_navbtn:
                {
                        FileRun("applications/nav.app","/");
                        return true;
                }
                break;

		    case BTN_USR:
                {
                        TPoint p;
                        PointAssign(&p,WIDGET(GSUSR)->Absolute.a.x-5, WIDGET(GSUSR)->Absolute.b.y-33);
                        PopUpMenu(&Me, p, Menu3, WIDGET(GSUSR), 1);
                        return true;
                }
                break;

		    case BTN_CP:
                {
                        FileRun("applications/nav.app","/Control Panel/");
                        return true;
                }
                break;

                case IDM_QUIT:
                {

                        if ( MessageBox(&Me, "Quit", "Are you really sure you want to Quit Aura?", MBB_YESNO|MBI_QUESTION) == MSG_YES)
                        {
			        TEvent Event;
								
                        	EventAssign(&Event,EV_MESSAGE, MSG_QUIT, NULL, KState, Mouse->State);
				HandleEvent(&Event);

                        }
                        return true;
                }
                break;

                /**
                *       Start menu items
                */
                case IDM_PROGRAMS: 
                {
                        FileRun(MENUITEM(Ev->Extra)->Args,NULL);
                        return true;
                }
                break;

                case IDM_CP:
                {
                        FileRun("xapps/nav.app","/Control Panel/");
                        return true;
                }
                break;

                case IDM_NAV:
                {
                        FileRun("applications/nav.app","/applications/");
                        return true;
                }
                break;

                case IDM_ABOUT:
                {
                        FileRun("applications/rundos.app", "/v/prompt.bat");
                        return true;
                }
                break;

                case IDM_doslynx:
                {
                        FileRun("applications/rundos.app", "/v/applications/doslynx/doslynxs.exe");
                        return true;
                }
                break;
		
                case IDM_dillo:
                {
                        FileRun("applications/rundos.app", "/v/applications/dillodos/dillo.bat");
                        return true;
                }
                break;

                case IDM_ftp:
                {
                        FileRun("applications/rundos.app", "/v/applications/2net/2ftp.bat");
                        return true;
                }
                break;
		
	}

        return false;
}


	TRect r, wr;
	PButton b;
	PLabel l;


//jmp_buf MainPoint;
l_int Main ( int argc, l_text *argv )
{

//	Export symbols		//
        APPEXPORT(TraybarAdd);
		APPEXPORT(Authenticate);
        APPEXPORT(TraybarRemove);
        APPEXPORT(WallpaperReLoad);
	stream = LoadData2("/System/sounds/welcome.mp3",DTYPE_SOUNDSTREAM);


//	Get desktop theme	//
        ThemeLoad(KeyGetText("/USER/DESKTOP/theme", "./DESKTOP/plex.ini"));


//      Initialize Wallpaper	//
        WallpaperInit();

//      Initialize Volume Settings	//
	VolumeInit();

	
	FileRun("applications/welcome.app",NULL);
	VolumeInit();
	        //setjmp(MainPoint); // Secure return point
	while (!LoggedIn)
       {
				SysPoll();
		#ifdef _OZONE__DJGPP_
                __dpmi_yield();
		#endif
        }
	

//      Initialize Panels	//
        GSPanelInit();

//	Initialize Volume Traybar 	//
	VolumeTrayInit();


	PlaySoundStream(stream);

        ClockTimer = NewTimer(&Me, 1000*10, &RefreshPanelClock, NULL);

       	  Menu2 = NewMenu(
     NewMenuItemEx("Games",NULL,0,0,NULL,&GenerateSubMenu,"SYSTEM/MENU/games",
     NewMenuItemEx("Media",NULL,0,0,NULL,&GenerateSubMenu,"SYSTEM/MENU/media",
     NewMenuItemEx("Office",NULL,0,0,NULL,&GenerateSubMenu,"SYSTEM/MENU/tools", 
     NewMenuItem( "DosLynx", NULL, IDM_doslynx, NULL, NULL,
     NewMenuItem( "Dillo v2", NULL, IDM_dillo, NULL, NULL,
     NewMenuItem( "FTP Server", NULL, IDM_ftp, NULL, NULL,
     NewMenuItem( "Prompt", NULL, IDM_ABOUT, NULL, NULL, NULL)))))))
        );

	  Menu3 = NewMenu(
     NewMenuItemEx("Documents",NULL,5,10,NULL,&GenerateSubMenu,"/system/User/Pictures",
     NewMenuItemEx("Network",NULL,5,10,NULL,&GenerateSubMenu,"/system/User/Music",
     NewMenuItemEx("Downloads",NULL,5,10,NULL,&GenerateSubMenu,"/system/User/Videos", NULL)))
        );

        Desktop->Insert = &DesktopWidgetInsert;
        Desktop->Remove = &DesktopWidgetRemove;
        Desktop->SetFirstEx = &DesktopWidgetSetFirstEx;
	return true;
}



void Close ( void )
{



        Desktop->Insert = &_WidgetInsert;
        Desktop->Remove = &_WidgetRemove;
        Desktop->SetFirstEx = &_WidgetSetFirstEx;

        GSPanelDestroy();
        GSWallpaperDestroy();
        KillTimer(ClockTimer);

	
}
