(*
////////////////////////////////////////////////////////////////////////////////
//
//  Combobox - Core File
//
//  (c) Copyright 2003 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
 .#include 'kernel.h'
 .#include 'combobox.h'
 .#include 'grfx.h'
*)
unit combobox;

interface

uses guitypes, grfx, widget;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$link grfx.o}
{$endif}

type 
  PCombobox = ^TCombobox;
  TCombobox = record

	o       :  TWidget;

	Items   : PList;
	
	Selected: PListviewItem;
	
	State   : l_uchar;
	OnUpdate: l_ulong;
	
  end;
  COMBOBOX = PCombobox;

function NewComboBox ( App: PApplication; r: TRect; OnUpdate: l_ulong ): PCombobox;
procedure ComboboxAddItem ( o: PCombobox; Caption: l_text; Icon: PIcon );

function ComboboxItemIndex 	( o: PCombobox; a: PListviewItem ): l_ulong;
procedure ComboboxSelectIndex ( o: PCombobox; Index: l_ulong );
procedure ComboboxSelectItem 	( o: PCombobox; p: PListviewItem );

implementation
{$ifndef LINK_CLIBS}

var
  AppVersion: l_ulong;
  SkinCombobox: PSkin;
  ComboboxClass: PWidgetClass;
  
const
  AppName: l_text   = 'Combobox widget';
  nUID: l_text      = 'combobox';
  NeededLibs: l_uid = ( 'widget','grfx','skin','listview','' );


procedure ComboboxDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
begin
	if ( UseSkins ) then
	begin
		DrawSkin(buffer, SkinCombobox, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);

		hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-4,(o^.Absolute.a.y+o^.Absolute.b.y)/2-2,o^.Absolute.b.x-SkinCombobox^.Right-11,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-5,(o^.Absolute.a.y+o^.Absolute.b.y)/2-1,o^.Absolute.b.x-SkinCombobox^.Right-10,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-6,(o^.Absolute.a.y+o^.Absolute.b.y)/2,o^.Absolute.b.x-SkinCombobox^.Right-9,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-7,(o^.Absolute.a.y+o^.Absolute.b.y)/2+1,o^.Absolute.b.x-SkinCombobox^.Right-8,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-8,(o^.Absolute.a.y+o^.Absolute.b.y)/2+2,o^.Absolute.b.x-SkinCombobox^.Right-7,COL_3DTEXT);

		if ( COMBOBOX(o)^.Selected )
			DrawNiceText (buffer, SkinCombobox^.Font, o^.Absolute.a.x+SkinCombobox^.Left, (o^.Absolute.a.y+o^.Absolute.b.y-text_height(SkinCombobox^.Font))/2, o^.Absolute.b.x-o^.Absolute.a.x-30,COMBOBOX(o)^.Selected^.Caption, SkinCombobox^.FontColor);
			//textout(buffer, SkinCombobox^.Font, COMBOBOX(o)^.Selected^.Caption,o^.Absolute.a.x+SkinCombobox^.Left, (o^.Absolute.a.y+o^.Absolute.b.y-text_height(SkinCombobox^.Font))/2, SkinCombobox^.FontColor);
	end
	else
	begin
		rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_3DFACE);

		if ( COMBOBOX(o)^.State )
			Rect3D(buffer,o^.Absolute.b.x-25,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DDARK,COL_3DLIGHT);
		else
			Rect3D(buffer,o^.Absolute.b.x-25,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);

		hline(buffer,o^.Absolute.b.x-5-4,(o^.Absolute.a.y+o^.Absolute.b.y)/2-2,o^.Absolute.b.x-5-11,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-5-5,(o^.Absolute.a.y+o^.Absolute.b.y)/2-1,o^.Absolute.b.x-5-10,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-5-6,(o^.Absolute.a.y+o^.Absolute.b.y)/2,o^.Absolute.b.x-5-9,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-5-7,(o^.Absolute.a.y+o^.Absolute.b.y)/2+1,o^.Absolute.b.x-5-8,COL_3DTEXT);
		hline(buffer,o^.Absolute.b.x-5-8,(o^.Absolute.a.y+o^.Absolute.b.y)/2+2,o^.Absolute.b.x-5-7,COL_3DTEXT);

		rectfill(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x-35,o^.Absolute.b.y, COL_BOXFACE);
		Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x-35,o^.Absolute.b.y, COL_BOXDARK,COL_BOXLIGHT);

		if ( COMBOBOX(o)^.Selected )
			DrawNiceText (buffer, default_font, o^.Absolute.a.x+5,(o^.Absolute.a.y+o^.Absolute.b.y-text_height(default_font))/2, o^.Absolute.b.x-o^.Absolute.a.x-35-5,COMBOBOX(o)^.Selected^.Caption, COL_3DTEXT);
		{	//textout(buffer, default_font, COMBOBOX(o)^.Selected^.Caption,o^.Absolute.a.x+8,(o^.Absolute.a.y+o^.Absolute.b.y-text_height(default_font))/2, COL_3DTEXT); }
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ComboboxEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  r: TRect;
  l: PListView;
  msg: l_ulong;
begin
	if (Ev^.Type = EV_MOUSE) then
	begin
		if ( Ev^.Message = WEvMouseLDown ) then
		begin
			COMBOBOX(o)^.State := 1;

			WidgetDraw(o,NIL);

			RectAssign(&r,o^.Absolute.a.x,o^.Absolute.b.y+1,o^.Absolute.b.x,o^.Absolute.b.y+min(100,ListviewEstimateHeight(COMBOBOX(o)^.Items)));

			l := CreateListview(o^.AppOwner,r,NIL,LVS_LIST,LVF_DRAWICONS);

			FreeList(l^.Items);

			l^.Items := COMBOBOX(o)^.Items;
			l^.TopItem := COMBOBOX(o)^.Selected;
			l^.OnValMsg := 1;

			InsertWidget(DeskTop,PWidget(l));

			WidgetDraw(PWidget(l), NIL);
			WidgetDraw(PWidget(SLLWDG(l)^.Vertical), NIL);

			ListviewSelectItem(l, GetSelectedItem(l), true, false);

			WidgetSetFocus(PWidget(l));

			if ( (msg = WidgetExecute(PWidget(l))) = 1 )
				COMBOBOX(o)^.Selected := GetSelectedItem(l);

			l^.Items := NewList();

			WidgetDispose(PWidget(SLLWDG(l)^.Vertical));
			SLLWDG(l)^.Vertical := NIL;
			WidgetDispose(PWidget(l));

			COMBOBOX(o)^.State := 0;

			WidgetDraw(o,NIL);

			if ( COMBOBOX(o)^.OnUpdate > 0 ) then WidgetSendSEvent(o, EV_MESSAGE, COMBOBOX(o)^.OnUpdate, NIL);

			ComboboxEventHandler := true;
		end;

	end else ComboboxEventHandler := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboboxSelectItem ( o: PCombobox; p: PListviewItem );
var a,b: PListItem;
begin
    a := o^.Items^.Last; b := a;
	if not assigned( a ) then exit;
	repeat
		if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED ) = LVI_SELECTED then
		begin
			PLISTVIEWITEM(a^.Data)^.Flags := PLISTVIEWITEM(a^.Data)^.Flags and not LVI_SELECTED;
		end;
		a := a^.Prev;
	until ( a = b );

	if assigned( p ) then
		p^.Flags := p^.Flags or LVI_SELECTED;
	

	o^.Selected	:= p;
	WidgetDraw(PWidget(o),NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboboxSelectIndex ( o: PCombobox; Index: l_ulong );
var a: PListviewItem;
begin
	if ( Index ) then 
	begin
		a := ListAt(o^.Items,Index);
		if assigned( a ) then ComboboxSelectItem(o,a);
	end else
		ComboboxSelectItem(o,NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ComboboxItemIndex ( o: PCombobox; a: PListviewItem ): l_ulong;
begin
	ComboBoxItemIndex := ListFoundIndex(o^.Items,a);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboboxAddItem ( o: PCombobox; Caption: l_text; Icon: PIcon );
var
  p: PListviewItem;
begin
    New(p);
	if not assigned(p) then ComboBoxAddItem := NIL else
	begin
	memset(p, 0, sizeof(TListviewItem));
	ListviewInitItem(LISTVIEWITEM(p),Caption,Icon);
	ListAdd (o^.Items, Caption, p, [(void * )}{$ifdef fpc}@{$endif}FreeListviewItem);
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboBoxFree ( o: PWidget );
begin
	//FreeList(PCOMBOBOX(o)^.Items);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewComboBox ( App: PApplication; r: TRect; OnUpdate: l_ulong ): PCombobox;
var 
  o: PComboBox;
begin
	NEW(o);
	fillchar(o,sizeof(TCombobox),0);

	IntialiseWidget(App,ComboboxClass,PWidget(o),r);

	o^.Items := NewList;

	PWidget(o)^.Draw         := {$ifdef fpc}@{$endif}ComboboxDraw;
	PWidget(o)^.EventHandler := {$ifdef fpc}@{$endif}ComboboxEventHandler;
	PWidget(o)^.FreeEx := {$ifdef fpc}@{$endif}ComboBoxFree;
	o^.OnUpdate := OnUpdate;
	NewComboBox := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function  StdCreateCombobox ( PApplication App, TRect r ): PWidget;
begin
	StdCreateComboBox := PWidget(NewComboBox( App, r, 0 ));
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
var
  r: TRect; cbx: PComboBox;
begin
	ComboboxClass := CreateWidgetClass( ULONG_ID('C','m','b','B'), StdWidgetClass, 0 );
	ComboboxClass^.StdCreate ::= {(void*)}{$ifdef fpc}@{$endif}StdCreateCombobox;

	
	
	/*TRect r := { 100,100,300,120 };
	PCombobox cbx := NewComboBox(&Me,r);

	ComboboxAddItem(cbx,'Item a',NIL,NIL);
	ComboboxAddItem(cbx,'Item b',NIL,NIL);
	ComboboxAddItem(cbx,'Item c',NIL,NIL);
	ComboboxAddItem(cbx,'Item d',NIL,NIL);
	ComboboxAddItem(cbx,'Item e',NIL,NIL);
	ComboboxAddItem(cbx,'Item f',NIL,NIL);

	InsertWidget(DeskTop,PWidget(cbx));

	WidgetDraw(PWidget(cbx),NIL);*/

	SkinCombobox := GetSkinItem('Combobox');

	APPEXPORT(NewComboBox);
	APPEXPORT(ComboboxAddItem);
	APPEXPORT(ComboboxItemIndex);
	APPEXPORT(ComboboxSelectIndex);
	APPEXPORT(ComboboxSelectItem);

	LibMain := true;
end;

procedure Close;
begin

end;


{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,0,0,1);
  SkinCombobox := NIL;

{$endif}  {LINK_CLIBS}
end.
