/////////////////////////////////////////////////////////////////////////////////
//
//	Textbox Core File 
//
//	Aura Milestone 3 
//
//	(c) Copyright 2012 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
unit textbox;
{
 .include "kernel.h"
 .include "widget.h"
 .include "textbox.h"
 .include "label.h"
 .include "scrollbar.h"
 .include "menu.h"
}
interface

uses
  guitypes,timer,dynld,metrics,text,widget,skin,ulabel,menu,scrllbar,grfx,font,
  allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK widget.o}
   {$LINK textbox.o}
   {$LINK label.o}
   {$LINK scrollbar.o}
   {$LINK menu.o}
{$endif}

type
  PTextbox = ^TTextbox;
  TTextbox = record

	o        : TScrollWidget;

	Font     :  PFONT;

	Flags    : l_ulong;

	Text     : l_text;

	SelPos   : l_ulong;
	SelPosEnd: l_ulong;

	DragPos  : l_ulong;

	ValidMsg : l_ulong;

	Tmr      : PTimer;

  end;

const
  TBF_EDITABLE		  = $0001;
  TBF_MULTILINE		  = $0002;
  TBF_TIMERHIDECURSOR =	$0004;
  TBF_PASSWORD	      =	$0008;
  TBF_DRAGING		  = $0010;
  TBF_PREDRAG		  = $0020;

function CreateTextbox(App:PApplication; r:TRect; Flags:l_ulong): PTextbox;
{$ifdef LINK_CLIBS}external;{$endif}

procedure TextBoxSetTextEx ( o:PTextbox; Txt:l_text );
{$ifdef LINK_CLIBS}external;{$endif}

procedure TextBoxSetText ( o:PTextbox; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
{$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
uses
{$ifdef OPascal}
  Sysutils
{$else}
  OSysutil,gdrivers
{$endif}
;
////////////////////////////////////////////////////////////////////////////////
var	AppVersion: l_ulong;
  
const  
  AppName: l_text = 'Textbox Widget';
  nUID: l_text = 'textbox';
  NeededLibs: array[0..6] of string = ( 'widget', 'clipbrd','grfx','skin', 'menu', 'scrllbar', '' );

const
  SkinTextbox: PSkin = NIL;
var TextboxClass: PWidgetClass;

////////////////////////////////////////////////////////////////////////////////
procedure TextBoxGetTextArea ( o:PTextbox; r:PRect );
begin
	if ( UseSkins ) then
		RectAssign(r^,
			PWIDGET(o)^.Absolute.a.x+SkinTextbox^.Left,
			PWIDGET(o)^.Absolute.a.y+SkinTextbox^.Top,
			PWIDGET(o)^.Absolute.b.x-SkinTextbox^.Right,
			PWIDGET(o)^.Absolute.b.y-SkinTextbox^.Bottom)
	else
		RectAssign(r^,
			PWIDGET(o)^.Absolute.a.x+2,
			PWIDGET(o)^.Absolute.a.y+2,
			PWIDGET(o)^.Absolute.b.x-2,
			PWIDGET(o)^.Absolute.b.y-2);
end;
////////////////////////////////////////////////////////////////////////////////
procedure Scroll ( o:PScrollWidget; p:TPoint );
begin
	if ( PWIDGET(o)^.Flags and WFForceBuffer ) = WFForceBuffer then
		WidgetDraw(PWIDGET(o),NIL)
	else begin
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags or WFForceBuffer;
		WidgetDraw(PWIDGET(o),NIL);
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags and not WFForceBuffer;
        end;
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var
  a,t: TRect;
  x,y,x1,x2: l_int;
  txt: l_text;
  pxt: l_ulong;
  axt: l_ulong;
  bxt: l_ulong;
  _end_: l_bool;
  ss,se,f: l_ulong;
begin
  TextBoxGetTextArea(PTEXTBOX(o), @a);
	
  if ( UseSkins ) then
	DrawSkin(buffer, SkinTextbox, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y)
  else begin
	al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_BOXFACE);
	Rect3D(buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y,COL_BOXLIGHT,COL_BOXDARK);
  end;
	
  GetClip(buffer,t);
  SetClip(buffer,a);
  al_set_clip_state(buffer, TRUE);

	
  if ( PTEXTBOX(o)^.Text<>'' ) then begin
    x := a.a.x-SLLWDG(o)^.Pos.x;
    y := (a.a.y)-SLLWDG(o)^.Pos.y;
    txt := PTEXTBOX(o)^.Text; pxt := x;
    f  := Length(txt); pxt := 0; { Anfangsposition auszugebender Text }
    ss := min(PTEXTBOX(o)^.SelPosEnd,PTEXTBOX(o)^.SelPos);
    se := max(PTEXTBOX(o)^.SelPosEnd,PTEXTBOX(o)^.SelPos);
    _end_ := false;
    while ( not _end_ ) do begin
      f := system.pos(#10,txt);
      if ( f = 0 ) then begin f := Length(txt); _end_ := true; end;
      if ( y >= w^.a.y - al_text_height(PTEXTBOX(o)^.Font)) and (y <= w^.b.y ) then begin
      if ( not( pxt > se ) or ( f < ss ) ) then begin
	if ( se = ss ) then begin
          if ((PTEXTBOX(o)^.Flags and TBF_TIMERHIDECURSOR)<>TBF_TIMERHIDECURSOR) then begin
	    x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[ss],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	    al_vline(buffer,x1,y,y+al_text_height(PTEXTBOX(o)^.Font),makecol(128,128,255));
	  end;
	end else begin
	  axt := max(ss,Length(txt)); bxt := min(se,f);
	  x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[axt],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	  x2 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[bxt],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	  al_rectfill(buffer,x1,y,x2,y+al_text_height(PTEXTBOX(o)^.Font),makecol(128,128,255));
	  if ( PTEXTBOX(o)^.Flags and TBF_TIMERHIDECURSOR ) = TBF_TIMERHIDECURSOR then begin
	    if ( not(pxt > PTEXTBOX(o)^.SelPosEnd) or (f < PTEXTBOX(o)^.SelPosEnd)) then begin
	      x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[PTEXTBOX(o)^.SelPosEnd],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	      al_vline(buffer,x1,y,y+al_text_height(PTEXTBOX(o)^.Font),makecol(255,255,255));
	    end;
	  end;
        end;
      end;
      if ( o^.Flags and WF_DRAGFOCUS ) = WF_DRAGFOCUS then
	if ( not( pxt > PTEXTBOX(o)^.DragPos ) or ( f < PTEXTBOX(o)^.DragPos ) ) then begin
          x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[PTEXTBOX(o)^.DragPos],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	  al_vline(buffer,x1,y,y+al_text_height(PTEXTBOX(o)^.Font),makecol(128,128,128));
	end;
	DrawTextEx(buffer,PTEXTBOX(o)^.Font,x,y,txt,txt[f],makecol(0,0,0),(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
      end;
      y := x + al_text_height(PTEXTBOX(o)^.Font)+1;
      pxt := f+1;
  end; End;
  SetClip(buffer,t);
  al_set_clip_state(buffer, TRUE);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxWriteChar ( o: PTextbox; c: l_char );
var
  Temp: l_text;
  Ex: l_text;
  ExPos: l_ulong;
  TxPos: l_ulong;
begin
        TxPos := 1;
        Temp := o^.Text[o^.SelPosEnd];
	SetLength(Ex,TextLen(o^.Text)+2+o^.SelPos-o^.SelPosEnd);
	Ex := System.Copy(o^.Text, 1, o^.SelPos);
        Ex := Ex + Temp;
        Temp := '';
        Ex[o^.SelPos] := c;

	o^.SelPos := o^.SelPos+1;
	o^.SelPosEnd := o^.SelPos;
	o^.DragPos := 1;
	o^.Text := Ex;
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxInserText ( o: PTextbox; Txt: l_text );
var
  l: l_ulong;
  Temp,Ex: l_text;
begin
  (*
	l := TextLen(Txt);
	l_text Temp = TextDup(o->SelPosEnd);
	l_text Ex = realloc(o->Text,TextLen(o->Text)+1+l+o->SelPos-o->SelPosEnd);
  *)
        system.Insert(Txt,o^.Text,o^.SelPos);
  (*
        TextCopy(Ex+(o->SelPos-o->Text),Txt);
	TextCopy(Ex+(o->SelPos-o->Text+l),Temp);
  	free(Temp);
	o->SelPos = Ex+(o->SelPos-o->Text+l);
	o->SelPosEnd = o->SelPos;
	o->DragPos = Ex;
	o->Text = Ex;
  *)
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDeleteSelection ( o: PTextbox );
var
  Temp,Ex: l_text; ExPos: l_ulong;
begin
	Temp := system.copy(o^.Text, 1, o^.SelPosEnd);
	{Ex = realloc(o->Text,TextLen(o->Text)+o->SelPos-o->SelPosEnd+1);}
	Ex := System.Copy(o^.Text, 1, o^.SelPos);
  	Temp := '';
  	if ( o^.DragPos >= o^.SelPos ) then begin
	  	if ( o^.DragPos <= o^.SelPosEnd ) then
	  		o^.DragPos := o^.SelPos
		else
	  		o^.DragPos := o^.SelPos + o^.DragPos-o^.SelPosEnd;
        end;
        {
	o^.DragPos = Ex+(o->DragPos-o->Text);
	o^.SelPos := Ex+(o->SelPos-o->Text);
        }
        o^.SelPosEnd := o^.SelPos;
	o^.Text := Ex;
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxGetSelection ( o: PTextbox ): l_text;
begin
   TextBoxGetSelection := TextNDup(o^.Text[o^.SelPos],o^.SelPosEnd-o^.SelPos);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDeleteMove ( o: PTextbox; Pos,EndPos: l_ulong );
var
  Temp,Ex: l_text;
begin
  if Length(o^.Text) >= o^.SelPosEnd+EndPos then
  Temp := System.Copy(o^.Text,EndPos,EndPos-o^.SelPosEnd);
  SetLength(Ex,TextLen(o^.Text)+abs(o^.SelPos-o^.SelPosEnd));
  Ex := system.copy(o^.Text,o^.SelPos,Pos-o^.SelPos);
  {Ex := Ex + System.Copy(o^.Text,abs(o^SelPos-pos));}
  Temp := '';
  o^.SelPos := o^.SelPos+Pos;
  o^.SelPosEnd := o^.SelPos;
  o^.DragPos := Longint(Addr(Ex));
  o^.Text := Ex;
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxCalculateMax ( o: PTextbox ): l_bool;
var
        a: TRect;
        Old: TPoint;
        x,y: Integer;
        t: l_text;
        _end_: l_bool;
        f: l_text;
        i: integer;
begin
        x := 0; y := 0; i := 1;

        Old := SLLWDG(o)^.Max;

	TextBoxGetTextArea(PTEXTBOX(o),@a);

	
	if ( o^.Text <> '' ) then begin
		t := o^.Text; f := t;
		_end_ := false;
		while ( not _end_ ) do
                begin
		   f := t[Length(t)]; {TextChr(t,'\n');}
		   if ( f = #10 ) then _end_ := true;
		   y := y + al_text_height(o^.Font)+1;
		   x := max(x,FontTextLengthEx(o^.Font,t,f,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD));
		   t := system.copy(t,i,length(t));
		end;
	end;

	if ( y > a.b.y-a.a.y ) then
		SLLWDG(o)^.Max.y := y - (a.b.y-a.a.y)
	else
		SLLWDG(o)^.Max.y := 0;

	if ( x > a.b.x-a.a.x ) then
		SLLWDG(o)^.Max.x := x - (a.b.x-a.a.x)
	else
		SLLWDG(o)^.Max.x := 0;

	if ( (Old.x = SLLWDG(o)^.Max.x) and (Old.y = SLLWDG(o)^.Max.y) ) then TextBoxCalculateMax := false;

	if ( SLLWDG(o)^.Pos.x > SLLWDG(o)^.Max.x  ) then ScrollBarSetPosX(SLLWDG(o),SLLWDG(o)^.Max.x);
	if ( SLLWDG(o)^.Pos.y > SLLWDG(o)^.Max.y  ) then ScrollBarSetPosY(SLLWDG(o),SLLWDG(o)^.Max.y);

	TextBoxCalculateMax := true;
end;
////////////////////////////////////////////////////////////////////////////////
function ReachStartOfLine ( t: l_text; l: l_ulong ): l_ulong;
begin
	Dec(l);
	while ( l > Length(t) ) do begin
		if ( t[l] = #10 ) then begin ReachStartOfLine := l+1; exit end else
		Dec(l);
	end;
	ReachStartOfLine := l;
end;
////////////////////////////////////////////////////////////////////////////////
procedure CheckSelVisibility ( o: PTextbox; RedrawScll: l_bool );
var a: TRect; x,y: l_long; t: l_text;
begin
	TextBoxGetTextArea(PTEXTBOX(o),@a);
	
	
	if ( SLLWDG(o)^.Max.x > 0 )  or ( SLLWDG(o)^.Max.y > 0 ) then begin
                t := system.copy(o^.Text,1,o^.SelPosEnd);
		LocatePointText(o^.Font,o^.Text,t,y,x,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
		x := x - SLLWDG(o)^.Pos.x;
		y := y - SLLWDG(o)^.Pos.y;
		if ( x < 0 ) then begin
			ScrollBarSetPosX(SLLWDG(o),max(0,SLLWDG(o)^.Pos.x+x));
			if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
		end else if ( x >= a.b.x-a.a.x ) then begin
			ScrollBarSetPosX(SLLWDG(o),min(SLLWDG(o)^.Max.x,SLLWDG(o)^.Pos.x+x-(a.b.x-a.a.x)));
			if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
		end;
		if ( y < 0 ) then begin
			ScrollBarSetPosY(SLLWDG(o),max(0,SLLWDG(o)^.Pos.y+y));
			if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
		end else begin
			y := y + al_text_height(o^.Font);
			if ( y >= a.b.y-a.a.y ) then begin
				ScrollBarSetPosY(SLLWDG(o),min(SLLWDG(o)^.Max.y,SLLWDG(o)^.Pos.y+y-(a.b.y-a.a.y)));
				if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
			end;
		end;
	end;
	
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxRefreshFromTo ( o: PTextbox; st,fn: l_text );
var
  t: l_text;
  r,a: TRect;
begin
  TextBoxGetTextArea(PTEXTBOX(o), @a);
	
  if ( Length(st) > Length(fn) ) then begin
		t := fn;
		fn := st;
		st := t;
  end;

  LocatePointText(o^.Font,o^.Text,st,r.a.y,r.a.x,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
  LocatePointText(o^.Font,o^.Text,fn,r.b.y,r.b.x,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
  r.b.x := r.b.x + FontCharLength(o^.Font,fn[1]);
  if ( r.a.y <> r.b.y ) then begin
		r.a.x := a.a.x;
		r.b.x := a.b.x;
	end else begin
		r.a.x := max(a.a.x-SLLWDG(o)^.Pos.x+r.a.x,a.a.x);
		r.b.x := min(a.b.x-SLLWDG(o)^.Pos.x+r.b.x,a.b.x);
        end;

  r.b.y += r.b.y + al_text_height(o^.Font);
  r.a.y := max(a.a.y-SLLWDG(o)^.Pos.y+r.a.y,a.a.y);
  r.b.y := max(a.b.y-SLLWDG(o)^.Pos.y+r.b.y,a.b.y);
  WidgetDraw(PWIDGET(o), @r);
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxDropAccept (
  o,Caller: PWidget;
  aType:l_ulong;
  var Data;
  SupportedMethods:l_ulong;
  var Method: l_ulong
): l_bool;
begin
  TextBoxDropAccept := false;
  if ((SupportedMethods and DMETHOD_MOVE)=DMETHOD_MOVE) and (aType = DTYPE_TEXT) then
    begin
      Method := DMETHOD_MOVE;
      TextBoxDropAccept := true;
    end;
    if ((SupportedMethods and DMETHOD_COPY)=DMETHOD_COPY) and (aType = DTYPE_TEXT) then
    begin
      Method := DMETHOD_COPY;
      TextBoxDropAccept := true;
    end;
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxDragBeforeDrop(o,Dest:PWidget; var Data; aType,Method:l_ulong): l_bool;
begin
  if ( o = Dest ) then
    if (PTEXTBOX(o)^.DragPos >= PTEXTBOX(o)^.SelPos) and
       (PTEXTBOX(o)^.DragPos <= PTEXTBOX(o)^.SelPosEnd) then
         TextBoxDragBeforeDrop := false;
	if ( Method = DMETHOD_MOVE ) then TextBoxDeleteSelection(PTEXTBOX(o));
	TextBoxDragBeforeDrop := true;
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDropData(o,Caller: PWidget; var Data; aType,Method:l_ulong);
begin
  if ( o = Caller ) then
    if ((PTEXTBOX(o)^.DragPos >= PTEXTBOX(o)^.SelPos) and
        (PTEXTBOX(o)^.DragPos <= PTEXTBOX(o)^.SelPosEnd)) then exit;
  PTEXTBOX(o)^.SelPos := PTEXTBOX(o)^.DragPos;
  PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.DragPos;
  TextBoxInserText(PTEXTBOX(o),l_text(Data));
  WidgetDraw(o, NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDragAfterDrop (o,Dest: PWidget; var Data; aType, Method: l_ulong);
begin
	if ( o <> Dest ) then
		WidgetDraw(o, NIL);
end;
////////////////////////////////////////////////////////////////////////////////
{function TextBoxEventHandler ( o: PWidget; Ev: PEvent ): l_bool;}
{
	
	if ( Ev->Type == EV_MESSAGE ){
		if ( Ev->Message == MSG_COPY ){
			if ( !( TEXTBOX(o)->Flags & TBF_PASSWORD ) )
			ClipboardPut(TYPE_TEXT,TextBoxGetSelection(TEXTBOX(o)),0);
		}
		else if ( (Ev->Message == MSG_CUT) && (TEXTBOX(o)->Flags & TBF_EDITABLE) ){
			if ( !( TEXTBOX(o)->Flags & TBF_PASSWORD ) )
			ClipboardPut(TYPE_TEXT,TextBoxGetSelection(TEXTBOX(o)),0);
			TextBoxDeleteSelection(TEXTBOX(o));
			WidgetDraw(o, NULL);
		}
		else if ( (Ev->Message == MSG_PASTE) && (TEXTBOX(o)->Flags & TBF_EDITABLE) ){
			TextBoxInserText(TEXTBOX(o),(l_text)ClipboardGet(TYPE_TEXT));
			WidgetDraw(o, NULL);
		}
		else if ( Ev->Message == MSG_DELETE ){
			TextBoxDeleteSelection(TEXTBOX(o));
			WidgetDraw(o, NULL);
		} else if ( Ev->Message == WEvNotifyFocused ){
			TEXTBOX(o)->Flags &= ~TBF_TIMERHIDECURSOR;
			WidgetDraw(o, NULL);
	 	} else if ( Ev->Message == WEvNotifyLostFocus ){
			TEXTBOX(o)->Flags |= TBF_TIMERHIDECURSOR;
			WidgetDraw(o, NULL);
		} else if ( Ev->Message == MSG_NOTIFY_AUTO_RESIZE ) {
			TextBoxCalculateMax(TEXTBOX(o));
			return true;
		}

		
		return true;
	}


	if ( Ev->Type == EV_MOUSE )
	{
		l_text t = TEXTBOX(o)->Text;
		
		TRect a;
		TextBoxGetTextArea(TEXTBOX(o),&a);
	
		if ( Ev->Message == WEvMouseLeft )
		{ 
			TEXTBOX(o)->Flags &= ~TBF_PREDRAG;
			return true;
		}
		if ( Ev->Message == MSG_DRAGCANCEL )
		{ 
			WidgetDraw(o, NULL);
			return true;
		}
	
		if ( (Ev->Message == WEvMouseLeft) && (o->Flags & WF_DRAGFOCUS) )
		{ 
			WidgetDraw(o, NULL);
			return true;
		}
		
		if ( (Ev->Message == WEvMouseMove) && (o->Flags & WF_DRAGFOCUS) )
		{ 
			l_text w = WhereInTextMultiLines(default_font,Mouse->State.p.x+SLLWDG(o)->Pos.x-a.a.x,Mouse->State.p.y+SLLWDG(o)->Pos.y-a.a.y,t,NULL,TEXTBOX(o)->Flags & TBF_PASSWORD);
			TEXTBOX(o)->DragPos = w;
			WidgetDraw(o, NULL);
			return true;
		}
		
		if ( Ev->Message == WEvMouseLDown )
		{
			l_text w = WhereInTextMultiLines(default_font,Mouse->State.p.x+SLLWDG(o)->Pos.x-a.a.x,Mouse->State.p.y+SLLWDG(o)->Pos.y-a.a.y,t,NULL,TEXTBOX(o)->Flags & TBF_PASSWORD);
			
			if ( (w > TEXTBOX(o)->SelPos) && (w <= TEXTBOX(o)->SelPosEnd) && ( TEXTBOX(o)->SelPosEnd != TEXTBOX(o)->SelPos )  ) {
				TEXTBOX(o)->Flags |= TBF_PREDRAG;
			} else {
				TEXTBOX(o)->SelPos = w;
				TEXTBOX(o)->SelPosEnd = w;
			}
			WidgetDraw(o, NULL);
			MasterWidget = o;
			return true;
		}
		if ( Ev->Message == WEvMouseLUp )
		{
			l_text w = WhereInTextMultiLines(default_font,Mouse->State.p.x+SLLWDG(o)->Pos.x-a.a.x,Mouse->State.p.y+SLLWDG(o)->Pos.y-a.a.y,t,NULL,TEXTBOX(o)->Flags & TBF_PASSWORD);
			/*if ( (TEXTBOX(o)->Flags & TBF_DRAGING) && (TEXTBOX(o)->Flags & TBF_EDITABLE) ) {
				TEXTBOX(o)->Flags &= ~TBF_DRAGING;
				if ( (TEXTBOX(o)->DragPos < TEXTBOX(o)->SelPos) || (TEXTBOX(o)->DragPos > TEXTBOX(o)->SelPosEnd) ) {
					l_text Temp = TextBoxGetSelection(TEXTBOX(o));
					TextBoxDeleteSelection(TEXTBOX(o));
					TEXTBOX(o)->SelPos = TEXTBOX(o)->DragPos;
					TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->DragPos;
					TextBoxInserText(TEXTBOX(o),Temp);
					free(Temp);
				}
			} else */if ( (TEXTBOX(o)->Flags & TBF_PREDRAG) ) {	
				TEXTBOX(o)->SelPos = w;
				TEXTBOX(o)->SelPosEnd = w;
				TEXTBOX(o)->Flags &= ~TBF_PREDRAG;
			} else {
				TEXTBOX(o)->SelPosEnd = max(TEXTBOX(o)->SelPos,w);
				TEXTBOX(o)->SelPos = min(TEXTBOX(o)->SelPos,w);
			}
			WidgetDraw(o, NULL);
			MasterWidget = NULL;
			return true;
		}
		if ( Ev->Message == WEvMouseMove && (Mouse->State.b & BUTTON_LEFT) ) {
			l_text w = WhereInTextMultiLines(default_font,Mouse->State.p.x+SLLWDG(o)->Pos.x-a.a.x,Mouse->State.p.y+SLLWDG(o)->Pos.y-a.a.y,t,NULL,TEXTBOX(o)->Flags & TBF_PASSWORD);
			/*if ( TEXTBOX(o)->Flags & TBF_DRAGING ) 
				TEXTBOX(o)->DragPos = w;
			else */if ( (TEXTBOX(o)->Flags & TBF_PREDRAG) && ( TEXTBOX(o)->SelPosEnd != TEXTBOX(o)->SelPos ) ) {	
				/*TEXTBOX(o)->DragPos = w;
				TEXTBOX(o)->Flags &= ~TBF_PREDRAG;
				TEXTBOX(o)->Flags |= TBF_DRAGING;*/
				TEXTBOX(o)->Flags &= ~TBF_PREDRAG;
				l_text t = TextBoxGetSelection(TEXTBOX(o));
				DragStart(o,DTYPE_TEXT,t,DMETHOD_COPY|DMETHOD_MOVE);
				free(t);
				MasterWidget = NULL;
			} else
				TEXTBOX(o)->SelPosEnd = w;
			WidgetDraw(o, NULL);
			CheckSelVisibility(TEXTBOX(o),( TEXTBOX(o)->Flags & TBF_MULTILINE ));
			return true;
		}

		if ( Ev->Message == WEvMouseRUp ) { // I love right click ! and you ??

			PMenu m = NewMenu(
     							NewMenuItem( "Copy", 		NULL, 	MSG_COPY, 	(TEXTBOX(o)->SelPos!=TEXTBOX(o)->SelPosEnd)&&(!(TEXTBOX(o)->Flags & TBF_PASSWORD ))?0:MI_DISABLE,NULL,
     							NewMenuItem( "Cut", 		NULL, 	MSG_CUT, 		(TEXTBOX(o)->SelPos!=TEXTBOX(o)->SelPosEnd)&&(TEXTBOX(o)->Flags & TBF_EDITABLE)&&(!(TEXTBOX(o)->Flags & TBF_PASSWORD ))?0:MI_DISABLE,NULL,
     							NewMenuItem( "Paste", 	NULL, 	MSG_PASTE, 	((TEXTBOX(o)->Flags & TBF_EDITABLE) && ClipboardGet(TYPE_TEXT,NULL))?0:MI_DISABLE,NULL,
     							NewMenuItemSeparator(
     							NewMenuItem( "Delete", 	NULL, 	MSG_DELETE, (TEXTBOX(o)->SelPos!=TEXTBOX(o)->SelPosEnd)&&(TEXTBOX(o)->Flags & TBF_EDITABLE)?0:MI_DISABLE,NULL,
     							NULL))))));

			PopUpMenu(o->AppOwner,Mouse->State.p,m,o,0);
			return true;
		}


	}

	if ( Ev->Type == EV_KEYBOARD )
	{
		if ( Ev->Keyb.Shift & KB_CTRL ) {
			if ( SCANCODE(Ev,KEY_C) ) {
				WidgetSendSEvent(o, EV_MESSAGE, MSG_COPY, NULL);
			}	else if ( SCANCODE(Ev,KEY_V) ) {
				WidgetSendSEvent(o, EV_MESSAGE, MSG_PASTE, NULL);
			}	else if ( SCANCODE(Ev,KEY_X) ) {
				WidgetSendSEvent(o, EV_MESSAGE, MSG_CUT, NULL);
			}
			return true;		
		}
		
		if ( SCANCODE(Ev, KEY_ESC) && TEXTBOX(o)->ValidMsg ) {
			WidgetSendSEvent(o, EV_MESSAGE, MSG_CANCEL, NULL);
			return true;
		}



		if ( SCANCODE(Ev,KEY_LEFT))
		{
			if ( Ev->Keyb.Shift & KB_SHIFT )
			{
				if ( TEXTBOX(o)->SelPosEnd > TEXTBOX(o)->Text ) TEXTBOX(o)->SelPosEnd--;
			}
			else
			{
				if ( TEXTBOX(o)->SelPos > TEXTBOX(o)->Text ) TEXTBOX(o)->SelPos--;
				TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
			}
		}
		else if ( SCANCODE(Ev,KEY_UP) && TEXTBOX(o)->Flags & TBF_MULTILINE )
		{
			l_text SOL = ReachStartOfLine(TEXTBOX(o)->Text,TEXTBOX(o)->SelPosEnd);
			if ( TEXTBOX(o)->Text != SOL ) {
				l_text sol = ReachStartOfLine(TEXTBOX(o)->Text,SOL-2);
				l_text pos = sol;
				if ( TEXTBOX(o)->SelPosEnd-SOL <= SOL-sol ) pos += TEXTBOX(o)->SelPosEnd-SOL;
				if ( Ev->Keyb.Shift & KB_SHIFT )
				{
					TEXTBOX(o)->SelPosEnd = pos;
				}
				else
				{
					TEXTBOX(o)->SelPos = pos;
					TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
				}
			}
		}
		else if ( SCANCODE(Ev,KEY_DOWN) && TEXTBOX(o)->Flags & TBF_MULTILINE )
		{
			l_text EOL = TextChr(TEXTBOX(o)->SelPosEnd,'\n');
			if ( EOL ) {
				l_text pos = EOL+1;
				l_text eol = TextChr(pos,'\n');
				l_text SOL = ReachStartOfLine(TEXTBOX(o)->Text,TEXTBOX(o)->SelPosEnd);
				if ( !eol ) eol = TEXTBOX(o)->Text+TextLen(TEXTBOX(o)->Text);
				if ( TEXTBOX(o)->SelPosEnd-SOL <= eol-EOL ) pos += TEXTBOX(o)->SelPosEnd-SOL;
				if ( Ev->Keyb.Shift & KB_SHIFT )
				{
					TEXTBOX(o)->SelPosEnd = pos;
				}
				else
				{
					TEXTBOX(o)->SelPos = pos;
					TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
				}
			}
		}
		else if ( SCANCODE(Ev,KEY_RIGHT) )
		{
			if ( Ev->Keyb.Shift & KB_SHIFT )
			{
				if ( TEXTBOX(o)->SelPosEnd < TEXTBOX(o)->Text+TextLen(TEXTBOX(o)->Text) ) TEXTBOX(o)->SelPosEnd++;
			}
			else
			{
				if ( TEXTBOX(o)->SelPos < TEXTBOX(o)->Text+TextLen(TEXTBOX(o)->Text) ) TEXTBOX(o)->SelPos++;
				TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
			}
		} else {

			if ( TEXTBOX(o)->SelPos > TEXTBOX(o)->SelPosEnd )
			{
				l_text t = TEXTBOX(o)->SelPosEnd;
				TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
				TEXTBOX(o)->SelPos = t;
			}

			if ( SCANCODE(Ev,KEY_BACKSPACE) && ( TEXTBOX(o)->SelPosEnd == TEXTBOX(o)->SelPos) && TEXTBOX(o)->Flags & TBF_EDITABLE  )
			{
				if ( TEXTBOX(o)->SelPos != TEXTBOX(o)->Text )
				{
					TextBoxDeleteMove(TEXTBOX(o),-1,0);
				}
			}
			else if (SCANCODE(Ev, KEY_HOME) && TEXTBOX(o)->Flags & TBF_EDITABLE)
			{
				TEXTBOX(o)->SelPos = TEXTBOX(o)->Text;
				TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
			}
			else if (SCANCODE(Ev, KEY_END) && TEXTBOX(o)->Flags & TBF_EDITABLE)
			{
				TEXTBOX(o)->SelPos = TEXTBOX(o)->Text;
				TEXTBOX(o)->SelPosEnd = TEXTBOX(o)->SelPos;
			}
			else if ( SCANCODE(Ev,KEY_DEL) && ( TEXTBOX(o)->SelPosEnd == TEXTBOX(o)->SelPos) && TEXTBOX(o)->Flags & TBF_EDITABLE )
			{
				if ( TEXTBOX(o)->SelPosEnd != (TEXTBOX(o)->Text+TextLen(TEXTBOX(o)->Text)) )
				{
					TextBoxDeleteMove(TEXTBOX(o),0,1);
				}
			}
			else if ( (SCANCODE(Ev,KEY_DEL) || SCANCODE(Ev,KEY_BACKSPACE)) && TEXTBOX(o)->Flags & TBF_EDITABLE ) {
		  	TextBoxDeleteSelection(TEXTBOX(o));
			}
			else if (SCANCODE(Ev, KEY_ENTER) && TEXTBOX(o)->Flags & TBF_EDITABLE )
			{

				if ( TEXTBOX(o)->Flags & TBF_MULTILINE )
					TextBoxWriteChar(TEXTBOX(o),'\n');
				else if ( TEXTBOX(o)->ValidMsg )
					WidgetSendSEvent(o, EV_MESSAGE, TEXTBOX(o)->ValidMsg, NULL);

			}
			else if ( ASCII(Ev) && TEXTBOX(o)->Flags & TBF_EDITABLE )
			{
				TextBoxWriteChar(TEXTBOX(o),ASCII(Ev));
				TextBoxRefreshFromTo(TEXTBOX(o),TEXTBOX(o)->SelPos-1,TEXTBOX(o)->SelPos);
			}

	 	}

		if ( TEXTBOX(o)->Flags & TBF_MULTILINE ){
		 	if ( TextBoxCalculateMax(TEXTBOX(o)) ) {
			 	CheckSelVisibility(TEXTBOX(o),true);
			 	WidgetDraw(WIDGET(SLLWDG(o)->Vertical), NULL);
	      WidgetDraw(WIDGET(SLLWDG(o)->Horizontal), NULL);
		 	} else
		 		CheckSelVisibility(TEXTBOX(o),true);
 		} else {
			TextBoxCalculateMax(TEXTBOX(o));
			CheckSelVisibility(TEXTBOX(o),false);


		}



		WidgetDraw(o, NULL);
		return true;
	}

	return ScrollWidgetEventHandler(o,Ev);
}
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxFreeEx ( o: PWidget );
begin
	if ( PTEXTBOX(o)^.Text <> '' ) then PTEXTBOX(o)^.Text := '';
	if assigned( PTEXTBOX(o)^.Tmr ) then KillTimer(PTEXTBOX(o)^.Tmr);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxSetTextEx ( o: PTextbox; Txt: l_text );
begin
  if ( o^.Text <> '' ) then o^.Text := '';
  o^.Text := Txt;
  o^.SelPos := 1;
  o^.SelPosEnd := 1;
  o^.DragPos := 1;
  if ( TextBoxCalculateMax(o) ) then begin
    if assigned( SLLWDG(o)^.Vertical ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
    if assigned( SLLWDG(o)^.Horizontal ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
  end;
  WidgetDraw(PWIDGET(o), NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxSetText(o:PTextbox; StrFormat:l_text; args:{$ifdef OPascal}array of const{$else}OpenArray{$endif});
var
  Buffer: l_text;
begin
  {$ifdef OPascal}
  Buffer := Format(StrFormat, args);
  {$else}
  FormatStr(Buffer, StrFormat, args);
  {$endif}
  TextBoxSetTextEx(o,Buffer);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxTimer ( o: Pointer );
begin
  if ((PTextBox(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE) and
     (((PWIDGET(o)^.Flags and WFFocused)=WFFocused) or
     ((PTextBox(o)^.Flags and TBF_TIMERHIDECURSOR)=TBF_TIMERHIDECURSOR)) then
  begin
    PTextBox(o)^.Flags := PTextBox(o)^.Flags xor TBF_TIMERHIDECURSOR;
    TextBoxRefreshFromTo(
       PTEXTBOX(o),
       PTextBox(o)^.Text[PTEXTBOX(o)^.SelPosEnd-1],
       PTextBox(o)^.Text[PTEXTBOX(o)^.SelPosEnd]);
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function CreateTextbox(App: PApplication; r: TRect; Flags: l_ulong): PTextbox;
var p: PTextbox; IsMultiline: l_ulong;
begin
  New(p);

  if not assigned(p) then CreateTextBox := NIL else
  begin
     if ((Flags and TBF_MULTILINE) <> TBF_MULTILINE) then begin
	if ( UseSkins ) then begin
          if ( r.b.x < r.a.y+al_text_height(default_font)+SkinTextbox^.Top+SkinTextbox^.Bottom ) then
	    RectAssign(&r,r.a.x,r.a.y,r.b.x,r.a.y+al_text_height(default_font)+SkinTextbox^.Top+SkinTextbox^.Bottom)
	  end else
	    if ( r.b.x < r.a.y+al_text_height(default_font)+4 ) then
	      RectAssign(r,r.a.x,r.a.y,r.b.x,r.a.y+al_text_height(default_font)+4);
     end;

     fillchar(p, sizeof(TTextbox), 0);

     p^.Font := default_font;
     p^.Flags := Flags or TBF_TIMERHIDECURSOR;
     p^.Text := '';
     { IntialiseWidget(App, PWIDGET(p), r, 'Textline'); }
     p^.SelPos := 1;
     p^.SelPosEnd := 1;
     p^.DragPos := 1;
     if (Flags and TBF_MULTILINE) = TBF_MULTILINE then IsMultiline := 3 else IsMultiline := 0;
     InitScrollWidget(SLLWDG(p), TextboxClass, App, r, IsMultiline);

     SLLWDG(p)^.Delta.y := al_text_height(p^.Font)+1;
     SLLWDG(p)^.Delta.x := 20;
     SLLWDG(p)^.Scrolled := {$ifdef fpc}@{$endif}Scroll;

     PWIDGET(p)^.Flags := PWIDGET(p)^.Flags or WFFocusable or WFForceBuffer;
     PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}TextBoxDraw;
     //PWIDGET(p)^.EventHandler := {$ifdef fpc}@{$endif}TextBoxEventHandler;
     PWIDGET(p)^.FreeEx := {$ifdef fpc}@{$endif}TextBoxFreeEx;

     PWIDGET(p)^.DropAccept 	:= {$ifdef fpc}@{$endif}TextBoxDropAccept;
     PWIDGET(p)^.DragBeforeDrop := {$ifdef fpc}@{$endif}TextBoxDragBeforeDrop;
     PWIDGET(p)^.DropData 	:= {$ifdef fpc}@{$endif}TextBoxDropData;
     PWIDGET(p)^.DragAfterDrop 	:= {$ifdef fpc}@{$endif}TextBoxDragAfterDrop;

     p^.Tmr := NewTimer(App, 500, {$ifdef fpc}@{$endif}TextBoxTimer, p );

     CreateTextBox := p;
  end;
end;

var tb:	PTextbox;
	
function TextboxSetRProp ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
var IsMultiLine: l_ulong;
begin
        TextboxSetRProp := false;
	if ( TextCompare(Name,'editable') = 0 ) then begin
		if ( l_bool(Value) ) then
		  PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_EDITABLE
		else
		  PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_EDITABLE;
		TextboxSetRProp := true;
	end;
	if ( TextCompare(Name,'multiline') = 0 ) then begin
		if ( l_bool(Value) ) then
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_MULTILINE
		else
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_MULTILINE;
		if (PTEXTBOX(o)^.Flags and TBF_MULTILINE) = TBF_MULTILINE
                   then IsMultiLine := 3
                else IsMultiLine := 0;
		ScrollBarChangeType(SLLWDG(o),IsMultiLine);
		
		TextboxSetRProp := true;
	end;
end;
	
function TextboxGetRProp ( o:PWidget; Name: l_text ): Pointer;
var tmp: l_bool;
begin
        TextboxGetRProp := NIL;
	if ( TextCompare(Name,'editable') = 0 ) then begin
		tmp := (PTEXTBOX(o)^.Flags and TBF_EDITABLE) = TBF_EDITABLE;
		TextboxGetRProp := DuplicateBool(tmp);
	end;
	if ( TextCompare(Name,'multiline') = 0 ) then begin
		tmp := (PTEXTBOX(o)^.Flags and TBF_MULTILINE) = TBF_MULTILINE;
		TextboxGetRProp := DuplicateBool(tmp);
	end;
end;

////////////////////////////////////////////////////////////////////////////////
function StdCreateTextbox ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateTextbox := PWIDGET(CreateTextbox( App, r, 0));
end;
////////////////////////////////////////////////////////////////////////////////
function LibMain ( argc: integer; argv: l_text ) : l_bool;
var r: TRect;
begin
	TextboxClass := CreateWidgetClass( ULONG_ID('T','x','t','B'), ScrollWidgetClass, 0 );
	TextboxClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateTextbox;
	TextboxClass^.SetRProp := {$ifdef fpc}@{$endif}TextboxSetRProp;
	TextboxClass^.GetRProp := {$ifdef fpc}@{$endif}TextboxGetRProp;

	WClassRegisterProp(TextboxClass,PTYPE_BOOL,'editable');
	WClassRegisterProp(TextboxClass,PTYPE_BOOL,'multiline');
	
	SkinTextbox := GetSkinItem('Textbox');
        (*
	APPEXPORT(CreateTextbox);
	APPEXPORT(TextBoxSetText);
	APPEXPORT(TextBoxSetTextEx);
        *)
	(*
	TRect r;
	WidgetSize(r,10,100,300,150),
	tb := CreateTextbox(@Me, r, TBF_EDITABLE of TBF_MULTILINE);
	InsertWidget(DeskTop, PWIDGET(tb));
	WidgetDraw(PWIDGET(tb), NIL);
	SLLWDG(tb)^.Scrolled := {$ifdef fpc}@{$endif}Scroll;
	TextBoxSetText(tb,
          'You see, my softwares works most of times...'+13#10+
          'I think this version works fine, but there is still work.'+#13#10+
          'Of course i will do it. So do not try to make changes in TextBox please.'+#13#10+
          'Anyway have a nice day using OUR GREAT sofware O3(oZone)'
        );
        *)
	LibMain := true;
end;
////////////////////////////////////////////////////////////////////////////////
procedure Close;
begin
	{WidgetDispose(PWIDGET(tb));}
end;
////////////////////////////////////////////////////////////////////////////////
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,1,1,0);

{$endif}  {LINK_CLIBS}
end.
