/**
*       Project:         Aura OS
*       
*       Codename:   Solar
*
*       Copyright 2008 Chase Finn and Finn Technologies.
*/
#include "kernel.h"
#include "widget.h"
#include "button.h"
#include "label.h"
#include "canvas.h"
#include "menu.h"
#include "list.h"
#include "skin.h"
#include "window.h"
#include "fms2.h"

#include "OZD/desktop.h"

#include <math.h>

l_ulong AppVersion = ULONG_ID(0,0,1,0);
char    AppName[] = "Desktop";
l_uid   nUID = "app:desktop";
l_uid NeededLibs[] = { "widget", "skin","button","label","menu","fms2","window","grfx","treeview","iodlg","" };

l_bool AppEventHandler ( PWidget o, PEvent Ev );

// Include the desktop files
#include "OZD/theme.c"
#include "OZD/wallpaper.c"
#include "OZD/panel.c"

void GenerateSubMenu ( PMenuItem o ,void* Args) {
        l_text n; p_bitmap i16; l_ulong  fi;

        PMenuItem m = NULL;
        PListItem a, b;
        PList l;


        l = ListDir(Args,0);
        if ( l ) {
                if ( l->Last )
                {
                        a = b = l->Last;
                        if ( a )
                        do
                        {
                                if ( FILEINFO(a->Data)->Name[0] != '.' ){
                                        if ( TextRChr(FILEINFO(a->Data)->Name,'.') )
                                                n = TextNDup(FILEINFO(a->Data)->Name,TextRChr(FILEINFO(a->Data)->Name,'.')-FILEINFO(a->Data)->Name);
                                        else
                                                n = TextDup(FILEINFO(a->Data)->Name);
                                        //FileGetIcons2(FILEINFO(a->Data),&i16,NULL,NULL,&fi);
                                        i16 = FileGetIcon2(FILEINFO(a->Data),16,&fi);
                                        m = NewMenuItemEx(n,i16,IDM_PROGRAMS,fi?MIF_FREEICON:0,NULL,NULL,TextDup(FILEINFO(a->Data)->VPathName),m );
                                        free(n);
                                }

                                a = a->Prev;
                        }
                        while ( a != b );
                }
                FreeList(l);
        }

        l = ListDir(Args,FA_DIREC|FA_LABEL);

        if ( l ) {;
                if ( l->Last )
                {
                        a = b = l->Last;
                        if ( a )
                        do
                        {
                                if ( FILEINFO(a->Data)->Attributes & FA_DIREC ) {
                                        //FileGetIcons2(FILEINFO(a->Data),&i16,NULL,NULL,&fi);
                                        i16 = FileGetIcon2(FILEINFO(a->Data),16,&fi);
                                        m = NewMenuItemEx(FILEINFO(a->Data)->Name,i16,0,0,NULL,&GenerateSubMenu,TextDup(FILEINFO(a->Data)->VPathName),m );
                                }
                                a = a->Prev;
                        }
                        while ( a != b );
                }
                FreeList(l);
        }

        o->SubMenu = NewMenu(m);

}



void DesktopWidgetInsert ( PWidget o, PWidget i )
{
        _WidgetInsert(o,i);

        if ( WidgetIsClassId(i,ULONG_ID('W','i','n',' ')) )
        {
                DebugMessage("TaskbarAdd %x",i);
                TaskbarAdd(WINDOW(i));
        }
}

void DesktopWidgetSetFirstEx ( PWidget o, PWidget i  )
{
        _WidgetSetFirstEx(o,i);

        if ( WidgetIsClassId(i,ULONG_ID('W','i','n',' ')) )
        {
                if (WINDOW(i)->WindowStates & WS_MINIMIZED)
                {
                        WINDOW(i)->WindowStates &=~ WS_MINIMIZED;
                }

                TaskbarRedraw(WINDOW(i));
        }
}


void DesktopWidgetRemove ( PWidget o, PWidget i )
{
        _WidgetRemove(o,i);

        if ( WidgetIsClassId(i,ULONG_ID('W','i','n',' ')) )
        {
                DebugMessage("TaskbarRemove %x...",i);
                TaskbarRemove(WINDOW(i));
                DebugMessage("...finished");
        }
}
PTimer  WelcomeTimer      = 0;
void WelcomeWidget()
{		        
		WelcomeTimer->Enable = 0;
		MessageBox(&Me, "Welcome", "Welcome To Aura", MBI_HARDWARE);
}

/**
*       Global variables
*/
PMenu Menu                      = 0;
PMenu Menu2                      = 0;
PMenu Menu3                      = 0;
PTimer  ClockTimer      = 0;
PTimer  IconsTimer      = 0;

l_bool MenuEventHandler ( PWidget o, PEvent Event )
{
        if ( Event->Type == EV_MESSAGE )
        {
                switch ( Event->Message )
                {
                        case MSG_IODLG_ITEMFILE_VAL:
                        {
                                if ( DIRVIEWITEM(Event->Extra)->f->Attributes & FA_DIREC )
                                {
                                        DirviewOpenDirectory(s,DIRVIEWITEM(Event->Extra)->f->VPathName);
                                }
                                else
                                {
                                        RunFile2(DIRVIEWITEM(Event->Extra)->f,0,NULL);
                                        return true;
                                }
                        }
                        break;

                        case MSG_TB_GOUP:
                        {
                                DirviewOpenParentDirectory(s);
                                TreeDirViewOpenDir(t,s->Directory);
                                return true;
                        }
                        break;
                }
        }

        return false;
}

void Livecd ( void )
{
        GSPanelInit();

        ClockTimer = NewTimer(&Me, 1000*30, &RefreshPanelClock, NULL);

       	  Menu2 = NewMenu(
     NewMenuItemEx("Games",NULL,0,0,NULL,&GenerateSubMenu,"SYSTEM/MENU/games",
     NewMenuItemEx("Media",NULL,0,0,NULL,&GenerateSubMenu,"SYSTEM/MENU/media",
     NewMenuItemEx("Office",NULL,0,0,NULL,&GenerateSubMenu,"SYSTEM/MENU/tools", 
     NewMenuItem( "Navigate", NULL, IDM_NAV, NULL, NULL, NULL))))
        );

	  Menu3 = NewMenu(
     NewMenuItemEx("Pictures",NULL,5,10,NULL,&GenerateSubMenu,"/system/User/Pictures",
     NewMenuItemEx("Music",NULL,5,10,NULL,&GenerateSubMenu,"/system/User/Music",
     NewMenuItemEx("Video",NULL,5,10,NULL,&GenerateSubMenu,"/system/User/Videos", NULL)))
        );

        Desktop->Insert = &DesktopWidgetInsert;
        Desktop->Remove = &DesktopWidgetRemove;
        Desktop->SetFirstEx = &DesktopWidgetSetFirstEx;

}

PWindow w	= 0;

l_bool AppEventHandler ( PWidget o, PEvent Ev )
{
        switch (Ev->Message)
        {
		    case BTN_APP:
                {
                 RectAssign(&r,WIDGET(GSApp)->Absolute.a.x, WIDGET(GSApp)->Absolute.a.y-293,WIDGET(GSApp)->Absolute.a.x+350, WIDGET(GSApp)->Absolute.a.y);
     		   GSSYSMENU = CreateWidget(&Me, r );
   		   GSSYSMENU->Draw = &SYSMENUDraw;
   		   GSSYSMENU->EventHandler = &PanelEventHandler;
      		   GSSYSMENU->Flags |= WFOnTop;
		   InsertWidget(DeskTop, GSSYSMENU);

	RectAssign(&r,1, 39, 348, 291);
        s = CreateDirview(&Me,r,"/",NULL,NULL);
        WIDGET(s)->AppEvHdl = &MenuEventHandler;
        WIDGET(s)->Flags |= WF_AUTORESIZE|WF_MAGNETRIGHT|WF_MAGNETBOTTOM;
        LISTVIEW(s)->Style = LVS_MOSC;
        InsertWidget(GSSYSMENU, WIDGET(s));

		   WidgetDrawAll(DeskTop);
                        return true;
                }
                break;

		    case BTN_navbtn:
                {
                        FileRun("applications/nav.app","/");
                        return true;
                }
                break;

		    case BTN_Ok:
                {
                        FileRun("applications/setup.app", NULL);
                        return true;
                }
                break;

		    case BTN_Ok2:
                {
			WidgetDispose ( WIDGET(w) );
                        Livecd();
                        return true;
                }
                break;

		    case BTN_USR:
                {
                        TPoint p;
                        PointAssign(&p,WIDGET(GSUSR)->Absolute.a.x-5, WIDGET(GSUSR)->Absolute.b.y-30);
                        PopUpMenu(&Me, p, Menu3, WIDGET(GSUSR), 1);
                        return true;
                }
                break;

		    case BTN_CP:
                {
                        FileRun("applications/nav.app","/Control Panel/");
                        return true;
                }
                break;

                case IDM_QUIT:
                {

                        if ( MessageBox(&Me, "Quit", "Are you really sure you want to Quit Aura?", MBB_YESNO|MBI_QUESTION) == MSG_YES)
                        {
                                TEvent Event;
                                EventAssign(&Event,EV_MESSAGE, MSG_QUIT, NULL, KState, Mouse->State);
                        HandleEvent(&Event);

                        }
                        return true;
                }
                break;

                /**
                *       Start menu items
                */
                case IDM_PROGRAMS: 
                {
                        FileRun(MENUITEM(Ev->Extra)->Args,NULL);
                        return true;
                }
                break;

                case IDM_CP:
                {
                        FileRun("xapps/nav.app","/Control Panel/");
                        return true;
                }
                break;

                case IDM_NAV:
                {
                        FileRun("applications/nav.app","/");
                        return true;
                }
                break;

                case IDM_ABOUT:
                {
                        FileRun("xapps/about.app","about");
                        return true;
                }
                break;
	}

        return false;
}


	TRect r, wr;
	PButton b;
	PLabel l;



l_int Main ( int argc, l_text *argv )
{
        /**
        *       Export symbols
        */
        APPEXPORT(TraybarAdd);
        APPEXPORT(TraybarRemove);
        APPEXPORT(WallpaperReLoad);


        /**
         *      Get desktop theme
         */

        ThemeLoad(KeyGetText("/USER/DESKTOP/theme", "./DESKTOP/plex.ini"));

        /**
         *      Initialize panels
         */
        WallpaperInit();

	RectAssign(&r,0, 0, 255, 90);
	w = CreateWindow( &Me, r, "Welcome", WF_FRAME|WF_CAPTION|WF_MINIMIZE|WF_CENTERED );
	InsertWidget(DeskTop, WIDGET(w));
	WIDGET(w)->AppEvHdl = &AppEventHandler;

	RectAssign(&r,5, 0, 250, 15);
	l = CreateLabel(&Me, r, "Welcome to Aura OS, ");
	WIDGET(l)->BackgroundColor = COL_3DFACE;
	InsertWidget(WIDGET(w), WIDGET(l));
	
	RectAssign(&r,5, 20, 250, 35);
	l = CreateLabel(&Me, r, "What would you like to do next?");
	WIDGET(l)->BackgroundColor = COL_3DFACE;
	InsertWidget(WIDGET(w), WIDGET(l));
	
	RectAssign(&r,5, 40, 250, 60);
	b = CreateButton(&Me,r,"Load the Live Desktop",BTN_Ok2);
	InsertWidget(WIDGET(w), WIDGET(b));
		
	RectAssign(&r,5, 65, 250, 85);
	b = CreateButton(&Me,r,"Install Aura to HDD",BTN_Ok);
	InsertWidget(WIDGET(w), WIDGET(b));
	
	WidgetDrawAll(WIDGET(w));
	return true;
}



void Close ( void )
{

        Desktop->Insert = &_WidgetInsert;
        Desktop->Remove = &_WidgetRemove;
        Desktop->SetFirstEx = &_WidgetSetFirstEx;

        GSPanelDestroy();
        GSWallpaperDestroy();
        KillTimer(ClockTimer);
	
}
